/*
 * outputs.c
 *
 * Created: 09.10.2017 10:29:07
 *  Author: dsosnowski
 */ 


#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <avr/pgmspace.h>
#include "defs.h"
#include "menu_defs.h"
#include "settings.h"
#include "dstemp.h"
#include "outputs.h"

//-----------------------------------
#define OUT34_PORT			PORTD
#define OUT34_DDR			DDRD
#define OUTPUT4				3
#define OUTPUT3				4

#define OUT12_PORT			PORTB
#define OUT12_DDR			DDRB
#define OUTPUT1				7
#define OUTPUT2				6

#define OUTPUTS_OFF			0
#define OUTPUTS_SET			1
#define OUTPUTS_NUM			4

#define OUTPUTS_DELAY		3

#define BUZZER_PORT			PORTD
#define BUZZER_DDR			DDRD
#define BUZZER_IN			PIND
#define BUZZER_PIN			2
#define BUZZER_SET			(BUZZER_PORT |= (1<<BUZZER_PIN))
#define BUZZER_OFF			(BUZZER_PORT &=~(1<<BUZZER_PIN))
#define BUZZER_GET			(BUZZER_IN & (1<<BUZZER_PIN))

#define VDET_PORT			PORTD
#define VDET_IN				PIND
#define VDET_PIN			5
#define VDET_GET			(VDET_IN & (1<<VDET_PIN))
#define VDET_DELAY			2

#define BEEP_PERIOD			(1500 / MAIN_DELAY)


uint8_t OutputsGetMenuIndex(uint8_t channel);
uint8_t OutputsGet(uint8_t channel);
void OutputsSet(uint8_t channel, uint8_t state);

//--------------------------------------------------------
struct{
	uint8_t state;
	uint8_t ext;
	uint8_t sensor;
	uint8_t silence;
	uint8_t beep;	
} alarm;


//--------------------------------------------------------
void OutputsInit(void) {
	OUT12_PORT &= ~(1<<OUTPUT1);
	OUT12_PORT &= ~(1<<OUTPUT2);
	OUT34_PORT &= ~(1<<OUTPUT3);
	OUT34_PORT &= ~(1<<OUTPUT4);
	
	OUT12_DDR |= (1<<OUTPUT1);
	OUT12_DDR |= (1<<OUTPUT2);
	OUT34_DDR |= (1<<OUTPUT3);
	OUT34_DDR |= (1<<OUTPUT4);
	
	OutputsSet(0,OUTPUTS_OFF);
	OutputsSet(1,OUTPUTS_OFF);
	OutputsSet(2,OUTPUTS_OFF);
	OutputsSet(3,OUTPUTS_OFF);
	
	BUZZER_PORT &= ~(1<<BUZZER_PIN);
	BUZZER_DDR |= (1<<BUZZER_PIN);
};

//--------------------------------------------------------
uint8_t OutputsGetMenuIndex(uint8_t channel) {
	if (channel > 3) channel = 3;
	return (MENU_INDEX_OUTPUT1_STATE + (channel * MENU_OUTPUT_GROUP_LEN));
}

//--------------------------------------------------------
uint8_t OutputsGet(uint8_t channel) {
	return (*SettValueSrc(OutputsGetMenuIndex(channel)));
}

//--------------------------------------------------------
void OutputsSet(uint8_t channel, uint8_t state) {
	if (state > 1) state = 1;
	*SettValueSrc(OutputsGetMenuIndex(channel)) = state;
}

//--------------------------------------------------------
void OutputsExe(void) {
	int16_t tOn;
	int16_t tOff;
	int16_t temp;
	uint8_t source;
	static uint8_t delayH[OUTPUTS_NUM];
	static uint8_t delayL[OUTPUTS_NUM];

	
	for(uint8_t i = 0; i<OUTPUTS_NUM; i++){
		source = *SettValueSrc(MENU_INDEX_OUTPUT1_CONDSRC + (i * MENU_OUTPUT_GROUP_LEN));
		
		if (source > SETT_SRC_MAN){
			tOn  = *SettValueSrc(MENU_INDEX_OUTPUT1_COND1P + (i*MENU_OUTPUT_GROUP_LEN));
			tOff = *SettValueSrc(MENU_INDEX_OUTPUT1_COND2P + (i*MENU_OUTPUT_GROUP_LEN));
			temp = TempGet(source);
			
			if (tOn >= tOff){
				if (OutputsGet(i) == 0) {
					if (temp >= tOn){
						if (delayH[i] < OUTPUTS_DELAY) {
							delayH[i]++;
						} else {
							OutputsSet(i, OUTPUTS_SET);
							delayL[i] = 0;
						}
					} else {
						delayH[i] = 0;
					}
				//if (OutputsGet(i) == 1) {
				} else {
					if (temp <= tOff){
						if (delayL[i] < OUTPUTS_DELAY) {
							delayL[i]++;
						} else {
							OutputsSet(i, OUTPUTS_OFF);
							delayH[i] = 0;
						}
					} else {
						delayL[i] = 0;
					}
				}
			//if (tOn < tOff){
			} else {
				if (OutputsGet(i) == 0) {
					if (temp <= tOn){
						if (delayH[i] < OUTPUTS_DELAY) {
							delayH[i]++;
						} else {
							OutputsSet(i, OUTPUTS_SET);
							delayL[i] = 0;
						}
					} else {
						delayH[i] = 0;
					}
					//if (OutputsGet(i) == 1) {
				} else {
					if (temp >= tOff){
						if (delayL[i] < OUTPUTS_DELAY) {
							delayL[i]++;
						} else {
							OutputsSet(i, OUTPUTS_OFF);
							delayH[i] = 0;
						}
					} else {
						delayL[i] = 0;
					}
				}
			}
		}
	}
}

//--------------------------------------------------------
void OutputsProc(void) {	
	static uint16_t beep = 0;
	
	if (OutputsGet(0) > OUTPUTS_OFF){
		OUT12_PORT |= (1<<OUTPUT1);
	} else {
		OUT12_PORT &= ~(1<<OUTPUT1);	
	}
	if (OutputsGet(1) > OUTPUTS_OFF){
		OUT12_PORT |= (1<<OUTPUT2);
	} else {
		OUT12_PORT &= ~(1<<OUTPUT2);
	}
	if (OutputsGet(2) > OUTPUTS_OFF){
		OUT34_PORT |= (1<<OUTPUT3);
	} else {
		OUT34_PORT &= ~(1<<OUTPUT3);
	}
	if (OutputsGet(3) > OUTPUTS_OFF){
		OUT34_PORT |= (1<<OUTPUT4);
	} else {
		OUT34_PORT &= ~(1<<OUTPUT4);
	}
	
	if (AlarmGet() > 0){
		if (alarm.silence == 0){
			if (alarm.beep < 0xFF) alarm.beep++;
			if (alarm.beep < 16){
				if (alarm.beep & 4) {
					BUZZER_SET;
				} else {
					BUZZER_OFF;
				}
			} else {
				alarm.sensor = 0;
				BUZZER_OFF;
			}			
		} else {
			BUZZER_OFF;
		}		
	} else {
		BUZZER_OFF;
		alarm.silence = 0;
	}	
}
	
//--------------------------------------------------------
void AlarmExe(void) {
	int16_t tH;
	int16_t tL;
	int16_t temp;
	uint8_t source;
	static uint8_t delayH;
	static uint8_t delayL;
	static uint8_t delayE;
	
	tH = *SettValueSrc(MENU_INDEX_BUZZER_COND1P);
	tL = *SettValueSrc(MENU_INDEX_BUZZER_COND2P);
	source = *SettValueSrc(MENU_INDEX_BUZZER_CONDSRC);
	temp = TempGet(source);
	
	if (alarm.state == 0){
		if ((temp >= tH) || (temp <= tL)){
			if (delayH < OUTPUTS_DELAY) {
				delayH++;
			} else {
				alarm.state = 1;
				delayL = 0;
			}
		} else {
			delayH = 0;
		}	
	} else {
		if ((temp < tH) && (temp > tL)){
			if (delayL < OUTPUTS_DELAY) {
				delayL++;
			} else {
				alarm.state = 0;
				delayH = 0;
			}
		} else {
			delayL = 0;
		}
	}
	
	if (*SettValueSrc(MENU_INDEX_BUZZER_VDET) > 0){
		if (alarm.ext == 0){
			if (VDET_GET > 0) {
				if (delayE < VDET_DELAY) {
					delayE++;
				} else {
					alarm.ext = 1;
				}
			} else {
				delayE = 0;
			}
		} else {
			if (VDET_GET == 0){
				alarm.ext = 0;
				delayE = 0;
			}
		}
	} else {
		alarm.ext = 0;	
	}
}

//--------------------------------------------------------
uint8_t AlarmGet(void) {
	if (alarm.ext > 0) return 3;
	if (alarm.state > 0) return 2;
	if (alarm.sensor > 0) return 1;	
	return 0;
}

//--------------------------------------------------------
void AlarmTick(void) {
	alarm.beep = 0;
}

//--------------------------------------------------------
void AlarmSensorSet(void) {
	alarm.sensor = 1;
}

//--------------------------------------------------------
void AlarmSilenceSet(void) {
	alarm.silence = 1;
}