/*
 * wzt.c
 *
 * Created: 19.09.2017 13:29:59
 *  Author: dsosnowski
 */ 


#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include "display.h"
#include "buttons.h"
#include "defs.h"
#include "menu.h"
#include "serial.h"
#include "settings.h"
#include "outputs.h"
#include "dstemp.h"
#include "menu_defs.h"
#include "wdog.h"

void MainHardInit(void);
void MainPeriodStart(void);
void MainPeriodProc(void);



uint8_t tempCycle = 0;
uint8_t tempErrorCycle = 0;
uint8_t extErrorCycle = 0;



//----------------------------------------------------------
int main(void)
{
	WdtSet(7);
	MainHardInit();
	SerialInit();
	DispInit();
	ButtonInit();
	SettInit();
	OutputsInit();
	MenuInit();
	DsInit();
	WdtReset();	
	sei();
	
	//DispBuffAddPosText("HELL-O!",0,0);
	DispNewMessage("     HELLO          AVT5665", 2);
	SerialDebug("Hello ", 1);
	DispBacklightSet(1);

		
	//-----------------------------------------------
    while(1)
    {
		MainPeriodStart();
		
		ButtonProc();
		
		if (ButtonActionGet() > 0){
			MenuMan(ButtonActionGet());
			MenuRefresh();
			AlarmSilenceSet();
		}
		
		if (tempCycle < (1000 / MAIN_DELAY)){
			tempCycle++;
			if (tempCycle >= (1000 / MAIN_DELAY)){
				tempCycle = 0;

				if (DsProcessNext() == DS_IDLE){
					DsProcess(0);
					DsProcess(1);
				}
				else
				{
					DsProcess(0);
					DsProcess(1);
					MenuRefresh();
					OutputsExe();
					MenuRefresh();					
				}
				
				if ((DsStatusGet(0) >= DS_NO_CONNECT) || (DsStatusGet(1) >= DS_NO_CONNECT)){
					tempErrorCycle++;
					if (tempErrorCycle > 6) {
						tempErrorCycle = 0;
						AlarmSensorSet();
						DispNewMessage("   AWARIA          CZUJNIKA", 3);
					}
				} else {
					tempErrorCycle = 0;
				}
				
				AlarmExe();
				if (AlarmGet() == 3){
					extErrorCycle++;
					if (extErrorCycle > 6){
						extErrorCycle = 0;
						DispNewMessage("    AWARIA         ZASILANIA", 3);						
					}
				} else {
					extErrorCycle = 0;
				}
				
				AlarmTick();
				MenuExe();
				DispExe();
				
			}
		} else {
			tempCycle = 0;			
		}
		
		OutputsProc();
		
		SettProc();
		MenuProcess();
		DispProcess();
		
		SerialWBufferSend();

		MainPeriodProc();
		WdtReset();
	}
}

//----------------------------------------------------------
void MainHardInit(void){
	ACSR = (1<<ACD);
	PRR = (1<<PRTWI | 1<<PRSPI | 1<<PRADC);
}

//----------------------------------------------------------
void MainPeriodStart(void){
	TCCR1B = 0;
	TCCR1A = 0;
	TCNT1 = 0;
	OCR1A = 156;
	TIFR1 = 0xFF;
	TCCR1B = 0x05;
}

//----------------------------------------------------------
void MainPeriodProc(void){
	while ((TIFR1 & (1<<OCF1A)) == 0){};
}


